#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <linux/types.h>
#include <linux/ioctl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <cairo/cairo.h>

/* Refer drivers/video/fbdev/fsl-dcu-fb.c */
#define MFB_SET_ALPHA       _IOW('M', 0, __u8)
#define MFB_GET_ALPHA       _IOR('M', 0, __u8)
#define MFB_SET_LAYER       _IOW('M', 4, struct layer_display_offset)
#define MFB_GET_LAYER       _IOR('M', 4, struct layer_display_offset)

#define GET_LAYER           1
#define SET_LAYER           2
#define GET_ALPHA           3
#define SET_ALPHA           4
#define SET_BITS_PER_PIXEL  5
#define GET_SCREENINFO      6
#define EXIT                7

/* Refer drivers/video/fbdev/fsl-dcu-fb.c */
struct layer_display_offset {
    int x_layer_d;
    int y_layer_d;
};

/*
 * Refer struct mfb_info in drivers/video/fbdev/fsl-dcu-fb.c
 * Layer 0: x_layer_d = 0; y_layer_d = 0,
 * Layer 1: x_layer_d = 50; y_layer_d = 50
 * Layer 2: x_layer_d = 100; y_layer_d = 100
 * Layer 3: x_layer_d = 150; y_layer_d = 150
 * Layer 4: x_layer_d = 200; y_layer_d = 200
 * Layer 5: x_layer_d = 250; y_layer_d = 250
 */

int main(void) {
    int fd;
    int ret;
    int choice;
    int layer_number;
    int bits_per_pixel;
    char frame_buffer_number;
    char fb_node[16] = {0};
    unsigned char alpha;
    struct layer_display_offset data;
    struct fb_var_screeninfo vscreeninfo;

    printf("Enter frame buffer number(0-5):\t");
    scanf("%c", &frame_buffer_number);
    sprintf(fb_node, "/dev/fb");
    fb_node[strlen(fb_node)] = frame_buffer_number;
    fb_node[strlen(fb_node)] = '\0';
    printf("Frame buffer node is: %s\n", fb_node);
    fd = open(fb_node, O_RDWR);
    if (fd < 0) {
        perror("Error");
        return -1;
    }

    while (1) {
        printf("\n1. Get layer\n");
        printf("2. Set layer\n");
        printf("3. Get alpha value\n");
        printf("4. Set alpha value\n");
        printf("5. Set bits per pixel\n");
        printf("6. Get screen information\n");
        printf("7. Exit\n");
        printf("\nEnter your choice (1-7):\t");
        scanf("%d[0-7]", &choice);

        switch(choice) {
        case GET_LAYER:
            ret = ioctl(fd, MFB_GET_LAYER, &data);
            if (ret)
                perror("Error");
            else
                if (data.x_layer_d == 0 && data.y_layer_d == 0)
                    printf("Current layer is 0\n");
                else if (data.x_layer_d == 50 && data.y_layer_d == 50)
                    printf("Current layer is 1\n");
                else if (data.x_layer_d == 100 && data.y_layer_d == 100)
                    printf("Current layer is 2\n");
                else if (data.x_layer_d == 150 && data.y_layer_d == 150)
                    printf("Current layer is 3\n");
                else if (data.x_layer_d == 200 && data.y_layer_d == 200)
                    printf("Current layer is 4\n");
                else if (data.x_layer_d == 250 && data.y_layer_d == 250)
                    printf("Currnet layer is 5\n");
            break;
        case SET_LAYER:
            layer_number = -1;
            printf("Enter the layer number(0-5) to set:\t");
            scanf("%d", &layer_number);
            switch (layer_number) {
            case 0:
                data.x_layer_d = 0;
                data.y_layer_d = 0;
                break;
            case 1:
                data.x_layer_d = 50;
                data.y_layer_d = 50;
                break;
            case 2:
                data.x_layer_d = 100;
                data.y_layer_d = 100;
                break;
            case 3:
                data.x_layer_d = 150;
                data.y_layer_d = 150;
                break;
            case 4:
                data.x_layer_d = 200;
                data.y_layer_d = 200;
                break;
            case 5:
                data.x_layer_d = 250;
                data.y_layer_d = 250;
                break;
            default:
                layer_number = -1;
                printf("Invalid layer number\n");
                break;
            }
            if (layer_number != -1) {
                ret = ioctl(fd, MFB_SET_LAYER, &data);
                if (ret)
                    perror("Error");
                else
                    printf("Layer set successfully\n");
            }
            break;
        case GET_ALPHA:
            alpha = -1;
            ret = ioctl(fd, MFB_GET_ALPHA, &alpha);
            if (ret)
                perror("Error");
            else
                printf("Alpha value is: %d\n", alpha);
            break;
        case SET_ALPHA:
            printf("Enter the value of alpha to set:\t");
            scanf("%d", &alpha);
            ret = ioctl(fd, MFB_SET_ALPHA, &alpha);
            if (ret)
                perror("Error");
            else
                printf("Alpha value set\n");
            break;
        case SET_BITS_PER_PIXEL:
                printf("\nEnter the value of bits per pixel:\t");
                scanf("%d", &bits_per_pixel);
                vscreeninfo.bits_per_pixel = bits_per_pixel;
                ret = ioctl(fd, FBIOPUT_VSCREENINFO, &vscreeninfo);
                if (ret)
                    perror("Error");
                else
                    printf("Bits per pixel set\n");
                break;
        case GET_SCREENINFO:
                ret = ioctl(fd, FBIOGET_VSCREENINFO, &vscreeninfo);
                if (ret)
                    perror("Error");
                else {
                    printf("\nVscreen Info:-\n");
                    printf(" Xres   = %4ld | Yres   = %4ld\n", vscreeninfo.xres,vscreeninfo.yres);
                    printf(" BPP    = %4ld | Height = %4ld | Width = %4ld\n", vscreeninfo.bits_per_pixel,
                           vscreeninfo.height,
                           vscreeninfo.width);
                    printf(" Xres_V = %4ld | Yres_V = %4ld\n", vscreeninfo.height,vscreeninfo.width);
                    printf(" Pixel format : RGBX_%ld%ld%ld%ld\n", vscreeninfo.red.length,
                           vscreeninfo.green.length,
                           vscreeninfo.blue.length,
                           vscreeninfo.transp.length);
                    printf(" Begin of bitfields(Byte ordering):-\n");    
                    printf("  Red    : %ld\n", vscreeninfo.red.offset); 
                    printf("  Blue   : %ld\n", vscreeninfo.blue.offset);
                    printf("  Green  : %ld\n", vscreeninfo.green.offset);
                    printf("  Transp : %ld\n", vscreeninfo.transp.offset);
              
                }
            break;
        case EXIT:
            close(fd);
            exit(0);
            break;
        default:
            printf("Invalid choice\n");
            choice = -1;
            break;
        }
    }
}
